@@ -19,6 +19,8 @@
  * Copyright (C) 2007 - 2008 Novell, Inc.
  */
 
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <sys/wait.h>
@@ -104,6 +106,7 @@ typedef struct {
 static void
 get_secrets_cb (NMSettingsConnection *connection,
                 guint32 call_id,
+                const char *agent_username,
                 const char *setting_name,
                 GError *error,
                 gpointer user_data)
@@ -121,7 +124,7 @@ get_secrets_cb (NMSettingsConnection *connection,
 guint32
 nm_act_request_get_secrets (NMActRequest *self,
                             const char *setting_name,
-                            guint32 flags,
+                            NMSettingsGetSecretsFlags flags,
                             const char *hint,
                             NMActRequestSecretsFunc callback,
                             gpointer callback_data)
@@ -214,8 +217,7 @@ nm_act_request_set_specific_object (NMActRequest *req,
 
 	priv = NM_ACT_REQUEST_GET_PRIVATE (req);
 
-	if (priv->specific_object)
-		g_free (priv->specific_object);
+	g_free (priv->specific_object);
 	priv->specific_object = g_strdup (specific_object);
 }
 
@@ -284,7 +286,7 @@ nm_act_request_get_default6 (NMActRequest *req)
 GObject *
 nm_act_request_get_device (NMActRequest *req)
 {
-	g_return_val_if_fail (NM_IS_ACT_REQUEST (req), FALSE);
+	g_return_val_if_fail (NM_IS_ACT_REQUEST (req), NULL);
 
 	return G_OBJECT (NM_ACT_REQUEST_GET_PRIVATE (req)->device);
 }
@@ -430,6 +432,8 @@ device_state_changed (NMDevice *device,
 	case NM_DEVICE_STATE_CONFIG:
 	case NM_DEVICE_STATE_NEED_AUTH:
 	case NM_DEVICE_STATE_IP_CONFIG:
+	case NM_DEVICE_STATE_IP_CHECK:
+	case NM_DEVICE_STATE_SECONDARIES:
 		new_ac_state = NM_ACTIVE_CONNECTION_STATE_ACTIVATING;
 		break;
 	case NM_DEVICE_STATE_ACTIVATED:
@@ -437,8 +441,12 @@ device_state_changed (NMDevice *device,
 		new_default = priv->is_default;
 		new_default6 = priv->is_default6;
 		break;
+	case NM_DEVICE_STATE_DEACTIVATING:
+		new_ac_state = NM_ACTIVE_CONNECTION_STATE_DEACTIVATING;
+		break;
 	default:
 		new_ac_state = NM_ACTIVE_CONNECTION_STATE_UNKNOWN;
+		new_default = new_default6 = FALSE;
 		break;
 	}
 
