@@ -52,7 +52,6 @@
 
 #include <arpa/inet.h>
 
-#define G_UDEV_API_IS_SUBJECT_TO_CHANGE
 #include <gudev/gudev.h>
 
 #define ENI_INTERFACES_FILE "/etc/network/interfaces"
@@ -601,9 +600,7 @@ update_system_hostname(NMInotifyHelper *inotify_helper,
 		return;
 	}
 
-	if (priv->hostname)
-		g_free(priv->hostname);
-
+	g_free(priv->hostname);
 	priv->hostname = g_strstrip(hostname_file);
 
 	/* We shouldn't return a zero-length hostname, but NULL */
